import { execSync } from 'child_process';
import chalk from 'chalk';
import spawn from 'cross-spawn';
import open from 'open';
import fs from 'fs';
import path from 'path';
import isGlob from 'is-glob';
import globSync from 'tiny-glob/sync';

const OSX_CHROME = "google chrome";
var Action;
(function(Action2) {
  Action2[Action2["NONE"] = 0] = "NONE";
  Action2[Action2["BROWSER"] = 1] = "BROWSER";
  Action2[Action2["SCRIPT"] = 2] = "SCRIPT";
})(Action || (Action = {}));
function getBrowserEnv() {
  const value = process.env.BROWSER;
  const args = process.env.BROWSER_ARGS ? process.env.BROWSER_ARGS.split(" ") : [];
  let action;
  if (!value) {
    action = 1;
  } else if (value.toLowerCase().endsWith(".js")) {
    action = 2;
  } else if (value.toLowerCase() === "none") {
    action = 0;
  } else {
    action = 1;
  }
  return { action, value, args };
}
function executeNodeScript(scriptPath, url) {
  const extraArgs = process.argv.slice(2);
  const child = spawn(process.execPath, [scriptPath, ...extraArgs, url], {
    stdio: "inherit"
  });
  child.on("close", (code) => {
    if (code !== 0) {
      console.log();
      console.log(chalk.red("The script specified as BROWSER environment variable failed."));
      console.log(`${chalk.cyan(scriptPath)} exited with code ${code}`);
      console.log();
    }
  });
  return true;
}
function startBrowserProcess(browser, url, args) {
  const shouldTryOpenChromiumWithAppleScript = process.platform === "darwin" && (typeof browser !== "string" || browser === OSX_CHROME);
  if (shouldTryOpenChromiumWithAppleScript) {
    const supportedChromiumBrowsers = [
      "Google Chrome Canary",
      "Google Chrome",
      "Microsoft Edge",
      "Brave Browser",
      "Vivaldi",
      "Chromium"
    ];
    for (const chromiumBrowser of supportedChromiumBrowsers) {
      try {
        execSync('ps cax | grep "' + chromiumBrowser + '"');
        execSync('osascript ../openChrome.applescript "' + encodeURI(url) + '" "' + chromiumBrowser + '"', {
          cwd: __dirname,
          stdio: "ignore"
        });
        return true;
      } catch (e) {
      }
    }
  }
  if (process.platform === "darwin" && browser === "open") {
    browser = void 0;
  }
  try {
    open(url, {
      app: browser ? {
        name: browser,
        arguments: args
      } : void 0,
      wait: false
    }).catch(() => {
    });
    return true;
  } catch (e) {
    return false;
  }
}
function openBrowser(url) {
  const { action, value, args } = getBrowserEnv();
  switch (action) {
    case 0:
      return false;
    case 2:
      return executeNodeScript(value, url);
    case 1:
      return startBrowserProcess(value, url, args);
    default:
      throw new Error("Not implemented.");
  }
}

var _a$1;
const DEV = "development";
const PROD = "production";
const NODE_ENV = (_a$1 = process.env.NODE_ENV) != null ? _a$1 : DEV;
const __DEV__ = NODE_ENV === DEV;
const __PROD__ = NODE_ENV === PROD;
const NODE_MODULES_REG = /[/\\]node_modules[/\\]/;
const CWD = process.cwd();
const EXTENSIONS = [".ts", ".tsx", ...Object.keys(require.extensions)];

const tryPkg = (pkg) => {
  try {
    return require.resolve(pkg);
  } catch (e) {
  }
};
const tryRequirePkg = (pkg) => {
  try {
    return require(pkg);
  } catch (e) {
  }
};
const isPkgAvailable = (pkg) => !!tryPkg(pkg);
const isTsAvailable = isPkgAvailable("typescript");
const isAngularAvailable = isPkgAvailable("@angular/core");
const isMdxAvailable = isPkgAvailable("@mdx/mdx") || isPkgAvailable("@mdx/react");
const isReactAvailable = isPkgAvailable("react");
const isSvelteAvailable = isPkgAvailable("svelte");
const isVueAvailable = isPkgAvailable("vue");
const tryFile = (filePath, includeDir = false) => {
  if (typeof filePath === "string") {
    return fs.existsSync(filePath) && (includeDir || fs.statSync(filePath).isFile()) ? filePath : "";
  }
  for (const file of filePath != null ? filePath : []) {
    if (tryFile(file, includeDir)) {
      return file;
    }
  }
  return "";
};
const tryExtensions = (filepath, extensions = EXTENSIONS) => {
  const ext = [...extensions, ""].find((ext2) => tryFile(filepath + ext2));
  return ext == null ? "" : filepath + ext;
};
const tryGlob = (paths, options = {}) => {
  const { absolute = true, baseDir = CWD } = typeof options === "string" ? { baseDir: options } : options;
  return paths.reduce((acc, pkg) => [
    ...acc,
    ...isGlob(pkg) ? globSync(pkg, {
      absolute,
      cwd: baseDir
    }) : [tryFile(path.resolve(baseDir, pkg), true)]
  ].filter(Boolean), []);
};
const identify = (_) => !!_;
const findUp = (searchEntry, searchFile = "package.json") => {
  console.assert(path.isAbsolute(searchEntry));
  if (!tryFile(searchEntry, true) || searchEntry !== CWD && !searchEntry.startsWith(CWD + path.sep)) {
    return "";
  }
  searchEntry = path.resolve(fs.statSync(searchEntry).isDirectory() ? searchEntry : path.resolve(searchEntry, ".."));
  do {
    const searched = tryFile(path.resolve(searchEntry, searchFile));
    if (searched) {
      return searched;
    }
    searchEntry = path.resolve(searchEntry, "..");
  } while (searchEntry === CWD || searchEntry.startsWith(CWD + path.sep));
  return "";
};
const arrayify = (...args) => args.reduce((arr, curr) => {
  arr.push(...Array.isArray(curr) ? curr : curr == null ? [] : [curr]);
  return arr;
}, []);

var _a, _b, _c, _d;
const pkg = (_a = tryRequirePkg(path.resolve("package.json"))) != null ? _a : {};
const lernaConfig = (_b = tryRequirePkg(path.resolve("lerna.json"))) != null ? _b : {};
const pkgsPath = (_d = (_c = lernaConfig.packages) != null ? _c : pkg.workspaces) != null ? _d : [];
const isMonorepo = Array.isArray(pkgsPath) && pkgsPath.length > 0;
const monorepoPkgs = isMonorepo ? tryGlob(pkgsPath) : [];

export { CWD, DEV, EXTENSIONS, NODE_ENV, NODE_MODULES_REG, PROD, __DEV__, __PROD__, arrayify, findUp, identify, isAngularAvailable, isMdxAvailable, isMonorepo, isPkgAvailable, isReactAvailable, isSvelteAvailable, isTsAvailable, isVueAvailable, monorepoPkgs, openBrowser, tryExtensions, tryFile, tryGlob, tryPkg, tryRequirePkg };
